;********************************************************************
;
;        JJY Radio Controlled Clock display control program
;
;                                             Device : PIC16F877A
;                                             Clock  : 4MHz
;                                             Author : Seiichi Inoue
;
;********************************************************************

        processor       16F877A
        list            f=INHX32
        include         p16f877a.inc
        radix           dec
        __config        h'3F72' ;OSC is HS
                                ;RB3,RB6,RB7 are I/O
                                ;Brown-out detection ON
                                ;Power-up timer ON
                                ;Code protection OFF
                                ;Data code protection OFF
                                ;Watchdog timer OFF
        errorlevel      -302    ;Eliminate bank warning
        errorlevel      -307    ;Eliminate Setting page bits warning

;************************  Label Definition  ************************
        cblock  h'20'
        led_10y                 ;10th of year
        led_1y                  ;1st of year
        led_10mth               ;10th of month
        led_1mth                ;1st of month
        led_10d                 ;10th of day
        led_1d                  ;1st of day
        led_10h                 ;10th of hour
        led_1h                  ;1st of hour
        led_10m                 ;10th of minute
        led_1m                  ;1st of minute
        led_10s                 ;10th of second
        led_1s                  ;1st of second
        led_wd                  ;Week

        led_ha                  ;Table head address 
        led_0                   ;7seg pattern 0
        led_1                   ;7seg pattern 1
        led_2                   ;7seg pattern 2
        led_3                   ;7seg pattern 3
        led_4                   ;7seg pattern 4
        led_5                   ;7seg pattern 5
        led_6                   ;7seg pattern 6
        led_7                   ;7seg pattern 7
        led_8                   ;7seg pattern 8
        led_9                   ;7seg pattern 9
        led_a                   ;7seg hyphen
        led_b                   ;7seg OFF

        scan_cont               ;scan counter
        stm                     ;Software timer
        clockst                 ;Clock status 0=NG,1=OK
        blink                   ;Blink status
        work_10h                ;10h work area
        work_1h                 ;1h work area
        last_10h                ;10h last look
        last_1h                 ;1h last look
        command                 ;0bit:STIM,1bit:DATE
        w_save                  ;W register save area
        s_save                  ;STATUS register save area

        rx_pnt                  ;Received data pointer
        rx_dat:20               ;Received data buffer(20 bytes)
        endc

seg7_ha equ     led_0           ;LED pattern table head address

seg7_0  equ     b'00111111'     ;-gfedcba Pattern 0
seg7_1  equ     b'00000110'     ;         Pattern 1
seg7_2  equ     b'01011011'     ;         Pattern 2
seg7_3  equ     b'01001111'     ;         Pattern 3
seg7_4  equ     b'01100110'     ;         Pattern 4
seg7_5  equ     b'01101101'     ;         Pattern 5
seg7_6  equ     b'01111101'     ;         Pattern 6
seg7_7  equ     b'00000111'     ;         Pattern 7
seg7_8  equ     b'01111111'     ;         Pattern 8
seg7_9  equ     b'01101111'     ;         Pattern 9
seg7_a  equ     b'01000000'     ;         Hyphen
seg7_b  equ     b'00000000'     ;         OFF

stimb   equ     0               ;STIM command send request bit
dateb   equ     1               ;DATE command send request bit

;************************  Program Start  ***************************
        org     0               ;Reset Vector
        goto    init
        org     4               ;Interrupt Vector
        goto    int

;************************  Initial process  *************************
init

;*** Port initialization
        bsf     status,rp0      ;Change to Bank1
        clrf    trisa           ;Set RA5-0 to OUTPUT
        clrf    trisb           ;Set RB7-0 to OUTPUT
        movlw   b'10000100'     ;Set RC7,2 to INPUT
        movwf   trisc           ;Set RC6-3,1-0 to OUTPUT
        clrf    trisd           ;Set RD7-0 to OUTPUT
        clrf    trise           ;Set RE2-0 to OUTPUT

;*** Set Option reg
        movlw   b'00000100'     ;PS=1:32
        movwf   option_reg      ;Set OPTION_REG

;*** ADCON1 set
        movlw   b'00000110'     ;Set all digital port
        movwf   adcon1          ;Set ADCON1
        bcf     status,rp0      ;Change to Bank0

;*** PORT set
        clrf    porta           ;Clear PORTA
        clrf    portb           ;Clear PORTB
        clrf    portc           ;Clear PORTC
        clrf    portd           ;Clear PORTD
        clrf    porte           ;Clear PORTE

;*** Set TMR0 (for LED scan display : 5mS)
        movlw   d'100'          ;256-5000/32us=100
        movwf   tmr0            ;Set TMR0

;*** Set TMR1 (for digit change : 500mS)
        movlw   b'00110001'     ;Pre=1:8 TMR1=Int TMR1=ON
        movwf   t1con           ;Set T1CON register
        movlw   h'0b'           ;65536-500000us/8us=3036
        movwf   tmr1h           ;Set TMR1H register
        movlw   h'dc'           ;3036=h'BDC'
        movwf   tmr1l           ;Set TMR1L register

;*** Set USART
        bsf     status,rp0      ;Change to Bank1
        movlw   d'25'           ;X=(4000kHz/(16*9.6kbps))-1=25
        movwf   spbrg           ;Set SPBRG reg (9600bps)
        movlw   b'00100110'     ;TX9=0,TXEN=1,SYNC=0,BRGH=1,TX9D=0
        movwf   txsta           ;Set TXSTA reg
        movlw   b'00100001'     ;RCIE=1,TXIE=0,TMR1IE=1
        movwf   pie1            ;Set PIE1 reg
        bcf     status,rp0      ;Change to Bank0
        movlw   b'10010000'     ;SPEN=1,RX9=0,CREN=1,RX9D=0
        movwf   rcsta           ;Set RCSTA reg

;*** Display data initialization
        movlw   h'0b'           ;Set OFF index
        movwf   led_10y         ;Write 10th of year
        movwf   led_1y          ;Write 1st of year
        movwf   led_10mth       ;Write 10th of month
        movwf   led_1mth        ;Write 1st of month
        movwf   led_10d         ;Write 10th of day
        movwf   led_1d          ;Write 1st of day
        movwf   led_10h         ;Write 10th of hour
        movwf   led_1h          ;Write 1st of hour
        movwf   led_10m         ;Write 10th of minute
        movwf   led_1m          ;Write 1st of minute
        movwf   led_10s         ;Write 10th of second
        movwf   led_1s          ;Write 1st of second
        movlw   7               ;Set light on data
        movwf   led_wd          ;Write week

        movlw   seg7_ha         ;Set pattern table head address
        movwf   led_ha          ;Save pattern table head address
        movlw   seg7_0          ;Set pattern 0
        movwf   led_0           ;Save pattern 0
        movlw   seg7_1          ;Set pattern 1
        movwf   led_1           ;Save pattern 1
        movlw   seg7_2          ;Set pattern 2
        movwf   led_2           ;Save pattern 2
        movlw   seg7_3          ;Set pattern 3
        movwf   led_3           ;Save pattern 3
        movlw   seg7_4          ;Set pattern 4
        movwf   led_4           ;Save pattern 4
        movlw   seg7_5          ;Set pattern 5
        movwf   led_5           ;Save pattern 5
        movlw   seg7_6          ;Set pattern 6
        movwf   led_6           ;Save pattern 6
        movlw   seg7_7          ;Set pattern 7
        movwf   led_7           ;Save pattern 7
        movlw   seg7_8          ;Set pattern 8
        movwf   led_8           ;Save pattern 8
        movlw   seg7_9          ;Set pattern 9
        movwf   led_9           ;Save pattern 9
        movlw   seg7_a          ;Set hyphen pattern
        movwf   led_a           ;Save hyphen pattern
        movlw   seg7_b          ;Set OFF pattern
        movwf   led_b           ;Save OFF pattern

;*** Work area initialization
        clrf    scan_cont       ;Scan counter
        clrf    clockst         ;Clock status
        clrf    blink           ;Blink status
        clrf    rx_pnt          ;Received data pointer
        clrf    work_10h        ;10h work area
        clrf    work_1h         ;1h work area
        clrf    last_10h        ;10h last look
        clrf    last_1h         ;1h last look
        clrf    command         ;Command request

;*** Set interrupt condition
        movlw   b'11100000'     ;GIE=ON,PEIE=ON,T0IE=ON
        movwf   intcon          ;Interruption enable

com01
        btfsc   command,stimb   ;STIM request ?
        goto    com03           ;Yes.
com02
        btfsc   command,dateb   ;DATE request ?
        goto    com04           ;Yes.
        goto    com01           ;Jump to STIM request check
com03
        call    stim            ;Send STIM command
        bcf     command,stimb   ;Clear STIM request
        goto    com02           ;Jump to DATE request check
com04
        call    date            ;Send DATE command
        bcf     command,dateb   ;Clear DATE request
        goto    com01           ;Jump to STIM request check

int
;********************** Interruption process ************************
        movwf   w_save          ;Save W register
        movfw   status          ;Read STATUS reg
        movwf   s_save          ;Save STATUS reg
        btfsc   pir1,rcif       ;Check RX int
        goto    rcv             ;Data receive
        btfsc   intcon,t0if     ;Check TMR0
        goto    led_display     ;TMR0 time-out(5mS)
        btfsc   pir1,tmr1if     ;Check TMR1IF
        goto    tmr1_out        ;TMR1 time-out(500mS)

int_end
;****************** END of interruption process *********************
        movfw   s_save          ;Read saved STATUS reg
        movwf   status          ;Recover STATUS reg
        swapf   w_save,f        ;Read saved W register
        swapf   w_save,w        ;Recover W register
        retfie

;******************* LED Display process (5mS) **********************
led_display
        bcf     intcon,t0if     ;Clear timer int flag
        movlw   d'100'          ;Set 5mS
        movwf   tmr0            ;Set TMR0

        movfw   scan_cont       ;Read scan counter
        btfsc   status,z        ;0 ?
        goto    led_cont0       ;Control 1s and 1d
        sublw   1
        btfsc   status,z        ;1 ?
        goto    led_cont1       ;Control 10s and 10d
        movfw   scan_cont       ;Read scan counter
        sublw   2
        btfsc   status,z        ;2 ?
        goto    led_cont2       ;Control 1m and 1mth
        movfw   scan_cont       ;Read scan counter
        sublw   3
        btfsc   status,z        ;3 ?
        goto    led_cont3       ;Control 10m and 10mth
        movfw   scan_cont       ;Read scan counter
        sublw   4
        btfsc   status,z        ;4 ?
        goto    led_cont4       ;Control 1h and 1y
        movfw   scan_cont       ;Read scan counter
        sublw   5
        btfsc   status,z        ;5 ?
        goto    led_cont5       ;Control 10h and 10y
        goto    led_conte1      ;Illegal

led_cont0
        movlw   b'00101101'     ;Set scan position
        movwf   porta           ;Write scan data
        btfss   portc,2         ;Lamp test ON ?
        goto    led_cont6       ;Yes
        movfw   led_1s          ;Read 1st of second digit
        addwf   led_ha,w        ;HA + Digit
        movwf   fsr             ;Set address
        movfw   indf            ;Read 7seg data
        movwf   portd           ;Set PORTD
        movfw   led_1d          ;Read 1st of day digit
        addwf   led_ha,w        ;HA + Digit
        movwf   fsr             ;Set address
        movfw   indf            ;Read 7seg data
        movwf   portb           ;Set PORTB
        movfw   led_wd          ;Read week data
        movwf   porte           ;Set PORTE
        goto    led_conte       ;Jump to display end

led_cont1
        movlw   b'00100100'     ;Set scan position
        movwf   porta           ;Write scan data
        btfss   portc,2         ;Lamp test ON ?
        goto    led_cont6       ;Yes
        movfw   led_10s         ;Read 10th of second digit
        addwf   led_ha,w        ;HA + Digit
        movwf   fsr             ;Set address
        movfw   indf            ;Read 7seg data
        movwf   portd           ;Set PORTD
        movfw   led_10d         ;Read 10th of day digit
        addwf   led_ha,w        ;HA + Digit
        movwf   fsr             ;Set address
        movfw   indf            ;Read 7seg data
        movwf   portb           ;Set PORTB
        goto    led_conte       ;Jump to display end

led_cont2
        movlw   b'00011011'     ;Set scan position
        movwf   porta           ;Write scan data
        btfss   portc,2         ;Lamp test ON ?
        goto    led_cont6       ;Yes
        movfw   led_1m          ;Read 1st of minute digit
        addwf   led_ha,w        ;HA + Digit
        movwf   fsr             ;Set address
        movfw   indf            ;Read 7seg data
        movwf   portd           ;Set PORTD
        movfw   led_1mth        ;Read 1st of month digit
        addwf   led_ha,w        ;HA + Digit
        movwf   fsr             ;Set address
        movfw   indf            ;Read 7seg data
        movwf   portb           ;Set PORTB
        goto    led_conte       ;Jump to display end

led_cont3
        movlw   b'00010010'     ;Set scan position
        movwf   porta           ;Write scan data
        btfss   portc,2         ;Lamp test ON ?
        goto    led_cont6       ;Yes
        movfw   led_10m         ;Read 10th of minute digit
        addwf   led_ha,w        ;HA + Digit
        movwf   fsr             ;Set address
        movfw   indf            ;Read 7seg data
        movwf   portd           ;Set PORTD
        movfw   led_10mth       ;Read 10th of month digit
        addwf   led_ha,w        ;HA + Digit
        movwf   fsr             ;Set address
        movfw   indf            ;Read 7seg data
        movwf   portb           ;Set PORTB
        goto    led_conte       ;Jump to display end

led_cont4
        movlw   b'00001001'     ;Set scan position
        movwf   porta           ;Write scan data
        btfss   portc,2         ;Lamp test ON ?
        goto    led_cont6       ;Yes
        movfw   led_1h          ;Read 1st of hour digit
        addwf   led_ha,w        ;HA + Digit
        movwf   fsr             ;Set address
        movfw   indf            ;Read 7seg data
        movwf   portd           ;Set PORTD
        movfw   led_1y          ;Read 1st of year digit
        addwf   led_ha,w        ;HA + Digit
        movwf   fsr             ;Set address
        movfw   indf            ;Read 7seg data
        movwf   portb           ;Set PORTB
        goto    led_conte       ;Jump to display end

led_cont5
        movlw   b'00000000'     ;Set scan position
        movwf   porta           ;Write scan data
        btfss   portc,2         ;Lamp test ON ?
        goto    led_cont6       ;Yes
        movfw   led_10h         ;Read 10th of hour digit
        addwf   led_ha,w        ;HA + Digit
        movwf   fsr             ;Set address
        movfw   indf            ;Read 7seg data
        movwf   portd           ;Set PORTD
        movfw   led_10y         ;Read 10th of year digit
        addwf   led_ha,w        ;HA + Digit
        movwf   fsr             ;Set address
        movfw   indf            ;Read 7seg data
        movwf   portb           ;Set PORTB
        goto    led_conte       ;Jump to display end

led_cont6                       ;Lamp test
        movlw   b'01111111'     ;Set digit 8
        movwf   portb           ;Set PORTB
        movwf   portd           ;Set PORTD
        movlw   d'7'            ;Set week matrix all ON
        movwf   porte           ;Set PORTE

led_conte
        incf    scan_cont,f     ;Scan counter + 1
        movfw   scan_cont       ;Read scan counter
        sublw   6
        btfsc   status,z        ;Count over ?
led_conte1
        clrf    scan_cont       ;Clear scan counter
        goto    int_end         ;Jump to interruption end

;***************** TMR1 time out process (500mS) ********************
tmr1_out
        bcf     pir1,tmr1if     ;Clear timer int flag
        movlw   h'0b'           ;Set 500mS
        movwf   tmr1h           ;Set TMR1H register
        movlw   h'dc'           ;
        movwf   tmr1l           ;Set TMR1L register

        btfsc   clockst,0       ;Check clock status
        goto    int_end         ;Working
        call    dcst            ;Send DCST command

        btfsc   blink,0         ;Check blink status
        goto    turn_off        ;Turn_off
turn_on
        movlw   h'0a'           ;Set hyphen index
        movwf   led_10y         ;Write 10th of year
        movwf   led_1y          ;Write 1st of year
        movwf   led_10mth       ;Write 10th of month
        movwf   led_1mth        ;Write 1st of month
        movwf   led_10d         ;Write 10th of day
        movwf   led_1d          ;Write 1st of day
        movwf   led_10h         ;Write 10th of hour
        movwf   led_1h          ;Write 1st of hour
        movwf   led_10m         ;Write 10th of minute
        movwf   led_1m          ;Write 1st of minute
        movwf   led_10s         ;Write 10th of second
        movwf   led_1s          ;Write 1st of second
        movlw   7               ;Set light on data
        movwf   led_wd          ;Write week
        bsf     blink,0         ;Set trun off
        goto    int_end         ;Jump to interruption end

turn_off
        movlw   h'0b'           ;Set OFF index
        movwf   led_10y         ;Write 10th of year
        movwf   led_1y          ;Write 1st of year
        movwf   led_10mth       ;Write 10th of month
        movwf   led_1mth        ;Write 1st of month
        movwf   led_10d         ;Write 10th of day
        movwf   led_1d          ;Write 1st of day
        movwf   led_10h         ;Write 10th of hour
        movwf   led_1h          ;Write 1st of hour
        movwf   led_10m         ;Write 10th of minute
        movwf   led_1m          ;Write 1st of minute
        movwf   led_10s         ;Write 10th of second
        movwf   led_1s          ;Write 1st of second
        movlw   7               ;Set light on data
        movwf   led_wd          ;Write week
        bcf     blink,0         ;Set turn_on
        goto    int_end         ;Jump to interruption end

;********************* Data receive process *************************
rcv
        bcf     pir1,rcif       ;Clear RX int flag
        btfss   rcsta,ferr      ;Frame error ?
        goto    rcv01           ;No error
        movf    rcreg,f         ;Received data clear
        clrf    clockst         ;Clear clock status
        goto    int_end         ;Jump to interruption end
rcv01
        btfss   rcsta,oerr      ;Over run ?
        goto    rcv02           ;Not over run
        bcf     rcsta,cren      ;Clear error flag
        bsf     rcsta,cren      ;Set Continuous Receive Enable bit
        clrf    clockst         ;Clear clock status
        goto    int_end         ;Jump to interruption end
rcv02
        movlw   rx_dat          ;Set head address of receive area
        addwf   rx_pnt,w        ;H.A + pinter
        movwf   fsr             ;Set save address
        movfw   rcreg           ;Read data
        movwf   indf            ;Save data
        sublw   h'0a'           ;Set 'LF' data
        btfsc   status,z        ;END of data ?
        goto    rcv03           ;Yes. 
        incf    rx_pnt,f        ;No. RX pointer + 1
        goto    int_end         ;Jump to interruption end

;*** Receive complete
rcv03
        btfsc   clockst,0       ;Check clock status
        goto    r_stim01        ;Working
        movfw   rx_dat          ;Read received 1st character
        sublw   'v'             ;Set check data
        btfsc   status,z        ;'Valid' ?
        goto    rcv04           ;Yes.
        clrf    rx_pnt          ;No. Clear RX pointer
        clrf    rx_dat          ;Clear 1st character area
        goto    int_end         ;Jump to interruption end

;*** Clock began to work
rcv04
        bsf     clockst,0       ;Set working status
        clrf    rx_pnt          ;Clear RX pointer
        clrf    rx_dat          ;Clear 1st character area
        bsf     command,dateb   ;DATE command send request
        bcf     clockst,1       ;Set DATE command finished
        goto    int_end         ;Jump to interruption end

;*** Get Hour, Minute, Second (Every correct second)
r_stim01
        btfss   clockst,1       ;Check command status
        goto    r_date01        ;DATE command
        movfw   rx_dat          ;Read received 1st character
        andlw   b'11110000'     ;Set check data
        sublw   h'30'           ;Check data
        btfsc   status,z        ;Character of number ?
        goto    r_stim02        ;Yes.
        clrf    rx_pnt          ;Clear RX pointer
        clrf    rx_dat          ;Clear 1st character area
        bsf     command,stimb   ;STIM command send request
        bsf     clockst,1       ;Set STIM command finished
        goto    int_end         ;Jump to interruption end

r_stim02
        movlw   rx_dat          ;Set head address of receive area
        addlw   7               ;Set 1st of second index
        movwf   fsr             ;Set address
        movfw   indf            ;Read 1st of second
        andlw   b'00001111'     ;Pick up number
        movwf   led_1s          ;Write 1st of second

        movlw   rx_dat          ;Set head address of receive area
        addlw   6               ;Set 10th of second index
        movwf   fsr             ;Set address
        movfw   indf            ;Read 10th of second
        andlw   b'00001111'     ;Pick up number
        movwf   led_10s         ;Write 10th of second

        movlw   rx_dat          ;Set head address of receive area
        addlw   4               ;Set 1st of minute index
        movwf   fsr             ;Set address
        movfw   indf            ;Read 1st of minute
        andlw   b'00001111'     ;Pick up number
        movwf   led_1m          ;Write 1st of minute

        movlw   rx_dat          ;Set head address of receive area
        addlw   3               ;Set 10th of minute index
        movwf   fsr             ;Set address
        movfw   indf            ;Read 10th of minute
        andlw   b'00001111'     ;Pick up number
        movwf   led_10m         ;Write 10th of minute

        movlw   rx_dat          ;Set head address of receive area
        addlw   1               ;Set 1st of hour index
        movwf   fsr             ;Set address
        movfw   indf            ;Read 1st of hour
        andlw   b'00001111'     ;Pick up number
        movwf   work_1h         ;Save 1st of hour

        movlw   rx_dat          ;Set head address of receive area
        movwf   fsr             ;Set address
        movfw   indf            ;Read 10th of hour
        andlw   b'00001111'     ;Pick up number
        movwf   work_10h        ;Save 10th of hour

;*** 24H to 12H display
r_stim03
        sublw   1               ;Set check data
        btfsc   status,z        ;1X ?
        goto    r_stim031       ;Yes.
        movfw   work_10h        ;Read 10th of hour
        sublw   2               ;Set check data
        btfsc   status,z        ;2X ?
        goto    r_stim033       ;Yes.
        movfw   work_1h         ;Read 1st of hour
        movwf   led_1h          ;Write 1st of hour
        movlw   h'0b'           ;Set OFF
        movwf   led_10h         ;Write 10th of hour
        goto    r_stim04
r_stim031
        movfw   work_1h         ;Read 1st of hour
        btfsc   status,z        ;10 o'clock ?
        goto    r_stim032       ;Yes.
        sublw   1               ;Set check data
        btfsc   status,z        ;11 o'clock ?
        goto    r_stim032       ;Yes.
        movfw   work_1h         ;Read 1st of hour
        sublw   2               ;Set check data
        btfsc   status,z        ;12 o'clock ?
        goto    r_stim032       ;Yes.
        movlw   2               ;Set data
        subwf   work_1h,w       ;1h - 2
        movwf   led_1h          ;Write 1st of hour
        movlw   h'0b'           ;Set OFF
        movwf   led_10h         ;Write 10th of hour
        goto    r_stim04
r_stim032
        movfw   work_1h         ;Read 1st of hour
        movwf   led_1h          ;Write 1st of hour
        movfw   work_10h        ;Read 10th of hour
        movwf   led_10h         ;Write 10th of hour
        goto    r_stim04
r_stim033
        movfw   work_1h         ;Read 1st of hour
        btfsc   status,z        ;20 o'clock ?
        goto    r_stim034       ;Yes.
        sublw   1               ;Set check data
        btfsc   status,z        ;21 o'clock ?
        goto    r_stim035       ;Yes.
        movfw   work_1h         ;Read 1st of hour
        sublw   2               ;Set check data
        btfsc   status,z        ;22 o'clock ?
        goto    r_stim036       ;Yes.
        movlw   1               ;Set data
        movwf   led_1h          ;Write 1st of hour
        movwf   led_10h         ;Write 10th of hour
        goto    r_stim04
r_stim034
        movlw   8               ;Set data
        movwf   led_1h          ;Write 1st of hour
        movlw   h'0b'           ;Set OFF
        movwf   led_10h         ;Write 10th of hour
        goto    r_stim04
r_stim035
        movlw   9               ;Set data
        movwf   led_1h          ;Write 1st of hour
        movlw   h'0b'           ;Set OFF
        movwf   led_10h         ;Write 10th of hour
        goto    r_stim04
r_stim036
        movlw   0               ;Set data
        movwf   led_1h          ;Write 1st of hour
        movlw   1               ;Set data
        movwf   led_10h         ;Write 10th of hour

;*** Date change check
r_stim04
        movfw   last_1h         ;Read last look 1h
        sublw   1               ;Set data
        btfss   status,z        ;x1 o'clock ?
        goto    r_stim05        ;No. Send STIM command
        movfw   last_10h        ;Read last look 10h
        sublw   1               ;Set data
        btfss   status,z        ;11 o'clock ?
        goto    r_stim05        ;No. Send STIM command
        movfw   led_1h          ;Read present 1h
        btfss   status,z        ;0 o'clock ?
        goto    r_stim05        ;No. Send STIM command
        bsf     command,dateb   ;DATE command send request
        bcf     clockst,1       ;Set DATE command finished
        goto    r_stim06        ;Save last look
r_stim05
        bsf     command,stimb   ;STIM command send request
        bsf     clockst,1       ;Set STIM command finished
r_stim06
        movfw   led_1h          ;Read 1h
        movwf   last_1h         ;Save 1h to last look
        movfw   led_10h         ;Read 10h
        movwf   last_10h        ;Save 10h to last look
        clrf    rx_pnt          ;Clear RX pointer
        clrf    rx_dat          ;Clear 1st character area
        goto    int_end         ;Jump to interruption end

;*** Get Year, Month, Day, Week
r_date01
        movfw   rx_dat          ;Read received 1st character
        andlw   b'11110000'     ;Set check data
        sublw   h'30'           ;Check data
        btfsc   status,z        ;Character of number ?
        goto    r_date02        ;Yes.
        clrf    rx_pnt          ;Clear RX pointer
        clrf    rx_dat          ;Clear 1st character area
        bsf     command,dateb   ;DATE command send request
        bcf     clockst,1       ;Set DATE command finished
        goto    int_end         ;Jump to interruption end

;*** Convert Week character to digit
r_date02
        movlw   rx_dat          ;Set head address of receive area
        addlw   11              ;Set week 1st character index
        movwf   fsr             ;Set address
        movfw   indf            ;Read week 1st character
        sublw   'M'             ;Check data
        btfsc   status,z        ;Monday ?
        goto    r_date021       ;Yes.
        movfw   indf            ;Read week 1st character
        sublw   'W'             ;Check data
        btfsc   status,z        ;Wednesday ?
        goto    r_date022       ;Yes.
        movfw   indf            ;Read week 1st character
        sublw   'F'             ;Check data
        btfsc   status,z        ;Friday ?
        goto    r_date023       ;Yes.
        movfw   indf            ;Read week 1st character
        sublw   'S'             ;Check data
        btfsc   status,z        ;Sunday or Saturday ?
        goto    r_date024       ;Yes.
        movfw   indf            ;Read week 1st character
        sublw   'T'             ;Check data
        btfsc   status,z        ;Tuesday or Thursday ?
        goto    r_date026       ;Yes.
        movlw   7
        goto    r_date028       ;Save week data
r_date021
        movlw   1               ;Set Monday data
        goto    r_date028       ;Save week data
r_date022
        movlw   3               ;Set Wednesday data
        goto    r_date028       ;Save week data
r_date023
        movlw   5               ;Set Friday data
        goto    r_date028       ;Save week data
r_date024
        movlw   rx_dat          ;Set head address of receive area
        addlw   12              ;Set week 2nd character index
        movwf   fsr             ;Set address
        movfw   indf            ;Read week 2nd character
        sublw   'U'             ;Check data
        btfsc   status,z        ;Sunday ?
        goto    r_date025       ;Yes.
        movlw   6               ;Set Saturday data
        goto    r_date028       ;Save week data
r_date025
        movlw   0               ;Set Sunday data
        goto    r_date028       ;Save week data
r_date026
        movlw   rx_dat          ;Set head address of receive area
        addlw   12              ;Set week 2nd character index
        movwf   fsr             ;Set address
        movfw   indf            ;Read week 2nd character
        sublw   'U'             ;Check data
        btfsc   status,z        ;Tuesday ?
        goto    r_date027       ;Yes.
        movlw   4               ;Set Thursday data
        goto    r_date028       ;Save week data
r_date027
        movlw   2               ;Set Tuesday data
r_date028
        movwf   led_wd          ;Write week

        movlw   rx_dat          ;Set head address of receive area
        addlw   9               ;Set 1st of day
        movwf   fsr             ;Set address
        movfw   indf            ;Read 1st of day
        andlw   b'00001111'     ;Pick up number
        movwf   led_1d          ;Write 1st of day

        movlw   rx_dat          ;Set head address of receive area
        addlw   8               ;Set 10th of day
        movwf   fsr             ;Set address
        movfw   indf            ;Read 10th of day
        andlw   b'00001111'     ;Pick up number
        btfss   status,z        ;Number = 0 ?
        goto    r_date03        ;No.
        movlw   h'0b'           ;Set OFF
r_date03
        movwf   led_10d         ;Write 10th of day

        movlw   rx_dat          ;Set head address of receive area
        addlw   6               ;Set 1st of month
        movwf   fsr             ;Set address
        movfw   indf            ;Read 1st of month
        andlw   b'00001111'     ;Pick up number
        movwf   led_1mth        ;Write 1st of month

        movlw   rx_dat          ;Set head address of receive area
        addlw   5               ;Set 10th of month
        movwf   fsr             ;Set address
        movfw   indf            ;Read 10th of month
        andlw   b'00001111'     ;Pick up number
        btfss   status,z        ;Number = 0 ?
        goto    r_date04        ;No.
        movlw   h'0b'           ;Set OFF
r_date04
        movwf   led_10mth       ;Write 10th of month

        movlw   rx_dat          ;Set head address of receive area
        addlw   3               ;Set 1st of year
        movwf   fsr             ;Set address
        movfw   indf            ;Read 1st of year
        andlw   b'00001111'     ;Pick up number
        movwf   led_1y          ;Write 1st of year

        movlw   rx_dat          ;Set head address of receive area
        addlw   2               ;Set 10th of year
        movwf   fsr             ;Set address
        movfw   indf            ;Read 10th of year
        andlw   b'00001111'     ;Pick up number
        movwf   led_10y         ;Write 10th of year

        clrf    rx_pnt          ;Clear RX pointer
        clrf    rx_dat          ;Clear 1st character area
        bsf     command,stimb   ;STIM command send request
        bsf     clockst,1       ;Set STIM command finished
        goto    int_end         ;Jump to interruption end

;******************** DCST command subroutine ***********************
dcst
        movlw   'D'             ;
        call    sout            ;Send out 'D'
        movlw   'C'             ;
        call    sout            ;Send out 'C'
        movlw   'S'             ;
        call    sout            ;Send out 'S'
        movlw   'T'             ;
        call    sout            ;Send out 'T'
        movlw   h'0d'
        call    sout            ;Send out CR
        movlw   h'0a'
        call    sout            ;Send out LF
        return

;******************** STIM command subroutine ***********************
stim
        movlw   'S'             ;
        call    sout            ;Send out 'S'
        movlw   'T'             ;
        call    sout            ;Send out 'T'
        movlw   'I'             ;
        call    sout            ;Send out 'I'
        movlw   'M'             ;
        call    sout            ;Send out 'M'
        movlw   h'0d'
        call    sout            ;Send out CR
        movlw   h'0a'
        call    sout            ;Send out LF
        return

;******************** DATE command subroutine ***********************
date
        movlw   'D'             ;
        call    sout            ;Send out 'D'
        movlw   'A'             ;
        call    sout            ;Send out 'A'
        movlw   'T'             ;
        call    sout            ;Send out 'T'
        movlw   'E'             ;
        call    sout            ;Send out 'E'
        movlw   h'0d'
        call    sout            ;Send out CR
        movlw   h'0a'
        call    sout            ;Send out LF
        return

;******************* Data send out subroutine ***********************
sout
        btfss   pir1,txif       ;Buffer full ?
        goto    sout            ;Yes. Wait
        movwf   txreg           ;No. Set data to TXREG register
        return

;********************************************************************
;                 END of Clock display check program
;********************************************************************

       end
